package Net::Z3950::DBIServer;

use DBI;
use Data::Dumper; # For debugging only
use Net::Z3950::SimpleServer;
use Net::Z3950::OID; # Provided by the SimpleServer package
use Net::Z3950::DBIServer::Config;
use Net::Z3950::DBIServer::ResultSet;
use Net::Z3950::DBIServer::XML;
use Net::Z3950::DBIServer::Exception;
use strict;

use vars qw($VERSION);
$VERSION = '0.01';


=head1 NAME

Net::Z3950::DBIServer - Generic Z39.50-to-RDBMS gateway module

=head1 SYNOPSIS

	use Net::Z3950::DBIServer;
	$handle = new Net::Z3950::DBIServer($configFile);
	$handle->launch_server("myAppName", @yazOptions);

=head1 DESCRIPTION

This module provides a simple gateway between the Z39.50 information
retrieval protocol and pretty much any SQL database you can think of.
When the DBIServer server module is running, you can connect
your favourite Z39.50 client to it, and issue searches.  The gateway
receives Z39.50 searches, translates them into SQL queries, executes
them against a relational database, translates the resulting and rows
of SQL tables into Z39.50 records, and sends them back out to the
client.

The mappings from Z39.50 concepts to SQL and back again are controlled
by tables described elsewhere.

=head1 METHODS

=head2 new()

	$handle = new Net::Z3950::DBIServer($configFile);

Creates and returns a new Z39.50-SQL gateway server, configured to
translate back and forth between Z39.50 and SQL concepts by the named
configuration file.  If the file is invalid, error messages are
printed to the standard error stream, and an undefined value is
returned.

=cut

### We need to stash our handle in a global due to interface misdesign.
my $horribleTemporaryGlobalHandle;

sub new {
    my $class = shift();
    my($configFile) = @_;

    my $config = new Net::Z3950::DBIServer::Config($configFile)
	or return undef;

    my $ss = new Net::Z3950::SimpleServer(INIT => \&init_handler,
					  SEARCH => \&search_handler,
					  FETCH => \&fetch_handler);

    # Delay DBI->open() until Init handler so each process gets its own
    $horribleTemporaryGlobalHandle = bless { ss => $ss,
			 config => $config,
			 rs => {}, # mapping of resultSetId to RS objects
		     }, $class;
    return $horribleTemporaryGlobalHandle;
}


sub init_handler {
    my $args = shift();

    $args->{IMP_ID} = "169"; # Mike Taylor's implementor ID
    $args->{IMP_NAME} = 'Net::Z3950::DBIServer';
    $args->{IMP_VER} = $VERSION;
    $args->{HANDLE} = $horribleTemporaryGlobalHandle;
    my $this = $args->{HANDLE};
    my $config = $this->{config};

    my $dbname = $config->dataSource();
    $this->{dbh} = DBI->connect($dbname)
	or die "can't open dataSource '$dbname': " . $DBI::errstr;
}


sub search_handler {
    my $args = shift();

    eval {
	_real_search_handler($args, @_);
    }; if ($@ && $@->isa('Net::Z3950::DBIServer::Exception')) {
	$args->{ERR_CODE} = $@->code();
	$args->{ERR_STR} = $@->addinfo();
    } elsif ($@) {
	die $@;
    }
}


sub _real_search_handler {
    my $args = shift();
    my $this = $args->{HANDLE};

    my $dbnames = $args->{DATABASES};
    ### Should SimpleServer (or something) provide constants for diagnostics?
    if (@$dbnames == 0) {
	# Specified combination of databases not supported ... not great!
	die new Net::Z3950::DBIServer::Exception(23);
    } elsif (@$dbnames > 1) {
	# Too many databases specified (addInfo = maximum)
	die new Net::Z3950::DBIServer::Exception(111, '1');
    }

    my $dbname = $dbnames->[0];
    my $config = $this->{config}->forDb($dbname);
    if (!defined $config) {
	# Database does not exist (addInfo = database name)
	# How is this different from 109 Database unavailable?
	die new Net::Z3950::DBIServer::Exception(235, $dbname);
    }

    my $rpn = $args->{RPN};
    my $attributeSet = $rpn->{attributeSet};
    my $query = $rpn->{query};
    my $SQLcond;
    $SQLcond = $query->SQLcond($this, $config->searchSpec(),
			       $attributeSet);

    # Would be nice if we could specify a particular set of fields
    # that we want instead of just saying "*", but we don't have a
    # single list that we can use: we have one per record syntax.
    my $rs;
    $rs = new Net::Z3950::DBIServer::ResultSet($this->{dbh}, "*",
					       $SQLcond, $config);

    my $setname = $args->{SETNAME};
    $this->{rs}->{$setname} = $rs;
    $args->{HITS} = $rs->count();
}


sub fetch_handler {
    my $args = shift();
    my $this = $args->{HANDLE};

    my $offset = $args->{OFFSET};
    my $setname = $args->{SETNAME};
    my $rs = $this->{rs}->{$setname};
    my $dataSpec = $rs->config()->dataSpec();
    #print Dumper($dataSpec);

    eval {
	my $hashref = $rs->fetch($offset-1);
	$args->{RECORD} = _format($hashref, $args->{REQ_FORM}, $dataSpec);
    }; if ($@ && $@->isa('Net::Z3950::DBIServer::Exception')) {
	$args->{ERR_CODE} = $@->code();
	$args->{ERR_STR} = $@->addinfo();
    } elsif ($@) {
	die $@;
    }
}


# PRIVATE to fetch_handler()
sub _format {
    my($hashref, $recsyn, $dataSpec) = @_;

    # I'm sure there's a more OO way to make this switch ...
    if ($recsyn eq Net::Z3950::OID::xml) {
	return Net::Z3950::DBIServer::XML::format($hashref,
						  $dataSpec->{XML});
#    } elsif ($recsyn eq Net::Z3950::OID::grs1) {
#	return Net::Z3950::DBIServer::GRS1::format($hashref,
#						   $dataSpec->{GRS1});
    } elsif ($recsyn eq Net::Z3950::OID::sutrs) {
	# Hardwired SUTRS formatting, for debugging only
	return join('', map { "$_: " . $hashref->{$_} . "\n" }
		    sort keys %$hashref);
    } else {
	die new Net::Z3950::DBIServer::Exception(238, "XML,SUTRS");
    }
}


# Here we just add the SQLcond() method to the class that we want.
# This is an unconventional alternative to a big switch statement in a
# single recursive SQLcond() function, but it's elegant and Perl
# allows it, so there.
#
# All these methods are in some sense private to our search handler.

package Net::Z3950::RPN::And;
sub SQLcond {
    my $this = shift();

    return "(" . ($this->[0]->SQLcond(@_) . ") AND (" .
		  $this->[1]->SQLcond(@_)) . ")";
}


package Net::Z3950::RPN::Or;
sub SQLcond {
    my $this = shift();

    return "(" . ($this->[0]->SQLcond(@_) . ") OR (" .
		  $this->[1]->SQLcond(@_)) . ")";
}


package Net::Z3950::RPN::AndNot;
sub SQLcond {
    my $this = shift();

    return "(" . ($this->[0]->SQLcond(@_) . ") AND NOT (" .
		  $this->[1]->SQLcond(@_)) . ")";
}


package Net::Z3950::RPN::Term;
sub SQLcond {
    my $this = shift();
    my($server, $config, $attributeSet) = @_;

    my %attributes;
    foreach my $attr (@{ $this->{attributes} }) {
	# There's no diagnostic for multiple attributes of the same
	# type, so we just ignore them :-)
	my $type = $attr->{attributeType};
	$attributes{$type} = $attr;
    }
    ### BIB-1/AA assumption (pretty much warranted! :-)
    my $use = $attributes{1};

    # Use attribute required but not supplied
    die new Net::Z3950::DBIServer::Exception(116)
	if !defined $use;
    my $val = $use->{attributeValue};
    $attributeSet = $use->{attributeSet} if defined $use->{attributeSet};

    # We now have an attribute set and an access point within it
    my $accessSpec = $config->accessPoint($attributeSet, $val);

    # Unsupported term value for Use attribute
    die new Net::Z3950::DBIServer::Exception(115, $val)
	if !defined $accessSpec;

    my $relation = relation($attributeSet, \%attributes);
    my $term = term($attributeSet, \%attributes, $this->{term});
    ###	We're completely ignoring position (3), structure (4) and
    #	completeness (5) attributes, in part because I don't honestly
    #	feel 100% that I know what they mean.
    if ($term =~ /%/) {
	### What does it mean to search (e.g.) < left-truncated term?
	die new Net::Z3950::DBIServer::Exception(123,
		"relation other than '=' with truncated term")
	    if $relation ne "=";
	$relation = "LIKE";
    }

    return $accessSpec->{columnname} . " " . $relation . " '" . $term . "'";
}


sub relation {
    my($attributeSet, $attributes) = @_;

    ### Should check for utility-set attributes as well as BIB-1
    my $attr = $attributes->{2};
    return "=" if !defined $attr;
    my $val = $attr->{attributeValue};

    my @rel = qw(< <= = >= > <>);
    my $rel = $rel[$val-1];
    if (!defined $rel) {
	# Maybe we should try harder on the others?
	#	100 = phonetic
	#	101 = stem
	#	102 = relevance
	#	103 = AlwaysMatches
	die new Net::Z3950::DBIServer::Exception(117, $val);
    }

    return $rel;
}


sub term {
    my($attributeSet, $attributes, $rawterm) = @_;

    ### Should check for utility-set attributes as well as BIB-1
    my $attr = $attributes->{5};
    return $rawterm if !defined $attr;
    my $val = $attr->{attributeValue};

    if ($val == 1) {
	return $rawterm . "%";
    } elsif ($val == 2) {
	return "%" . $rawterm;
    } elsif ($val == 3) {
	return "%" . $rawterm . "%";
    } elsif ($val == 100) {
	return $rawterm;
    } elsif ($val == 101) {
	$rawterm =~ tr/#/%/;
	return $rawterm;
    }

    # What do the others mean?  Can we implement them?
    #	102 = regExpr-1
    #	103 = regExpr-2
    die new Net::Z3950::DBIServer::Exception(120, $val);
}


# We now return you to your scheduled programmes ...
package Net::Z3950::DBIServer;


=head2 launch_server()

	$handle->launch_server("myAppName", @yazOptions);

Launches the Z39.50-SQL gateway server C<$handle>, using the specified
string as an identifier in any log messages, and with its Z39.50
server behavious controlled by C<@yazOptions> as described in the Yaz
manual at
http://www.indexdata.dk/yaz/doc/server.invocation.php
and also in
L<Net::Z3950::DBIServer::Run>

This method never returns unless an error occurs.

=cut

sub launch_server {
    my $this = shift();
    my($logIdent, @yazOptions) = @_;

    $this->{ss}->launch_server($logIdent, @yazOptions);
}


=head1 AUTHOR

Mike Taylor E<lt>mike@miketaylor.org.ukE<gt>

First version Saturday 2nd February 2002.

=head1 SEE ALSO

L<Net::Z3950::DBIServer::Spec>
describes the format of the configuration files which specify this
module's behaviour.

L<Net::Z3950::DBIServer::Config>
describes the API to the configuration file parser.

L<Net::Z3950::DBIServer::ResultSet>
describes the API to the internal representation of result sets.

L<Net::Z3950::DBIServer::XML>
describes the API to the XML record formatter.

L<Net::Z3950::DBIServer::Exception>
describes the simple exception objects used to represent Bib-1
diagnostics.

=cut


1;
