# $Header: /home/cvsroot/zSQLgate/DBIServer/Config.pm,v 1.3 2002/03/03 23:24:22 mike Exp $

package Net::Z3950::DBIServer::Config;
use IO::File;
use Carp;
use strict;


=head1 NAME

Net::Z3950::DBIServer::Config - Configuration parser for DBIServer

=head1 SYNOPSIS

	use Net::Z3950::DBIServer::Config;
	$config = new Net::Z3950::DBIServer::Config($configFile);

=head1 DESCRIPTION

This module parses configuration files for the
C<Net::Z3950::DBIServer> module.  I can't begin to imagine why anyone
who's not doing DBIServer development would be remotely interested in
the technical details of how that's done, and those people will be
reading the source rather than the documentation, so I don't propose
to write much.

The parser's a simple one-token-lookahead recursive descent parser,
similar to the one that's explained in some detail in pretty much the
first chapter of Aho Sethi & Ullman's ``New Dragon Book'',
I<Compilers: Principles, Techniques and Tools>.  If you want to write
parsers of any substance, you I<have> to have this book.  It is the
absolute bible of compilers.

It was a pleasure writing the parser in Perl - it took about a tenth
as long as similar ones that I've written in C in the past (i.e. a
couple of hours) and that includes the lexer which, to my delight,
turned out to be thirty (count 'em!) lines of code.

I'm not going to insult your intelligence by documenting the public
methods, since the only one is the constructor, whose synopsis above
tells you all you need to know except that the C<$configFile>
parameter is a fileI<name>, not a fileI<handle>.  The object that it
returns is pretty much self-documenting.  Print it with
C<Data::Dumper> if you don't believe me.

=cut


sub STRING     { 500 }
sub NUMBER     { 501 }
sub SYMBOL     { 502 }
sub DATABASE   { 503 }
sub SEARCH     { 504 }
sub ACCESS     { 505 }
sub ATTR       { 506 }
sub ATTRSET    { 507 }
sub DATA       { 508 }
sub COLUMN     { 509 }
sub FORMAT     { 510 }
sub GRS1       { 511 }
sub XML        { 512 }
sub RECORD     { 513 }
sub DATASOURCE { 514 }
sub CACHESIZE  { 515 }
sub TABLE      { 516 }


sub new {
    my $class = shift();
    my($filename) = @_;

    if (!defined $filename) {
	print STDERR "$class: configuration file not specified\n";
	return undef;	
    }

    my $fh = new IO::File("<$filename");
    if (!defined $fh) {
	print STDERR "$class: can't open '$filename': $!\n";
	return undef;	
    }

    ### Should probably be a class with accessor methods
    my $state = { class => $class,
		  filename => $filename,
		  fh => $fh,
		  buffer => '',
		  line => 0,
		  token => undef,
		  lexeme => undef };
    lex($state);
    return _parse_config($state);
}


# PRIVATE to the new() method
sub _parse_config {
    my($state) = @_;

    my $config = [];
    my $dataSource;
    while (defined $state->{token}) {
	if ($state->{token} == DATASOURCE) {
	    die "multiple dataSources specified"
		if defined $dataSource;
	    match($state, DATASOURCE);
	    match($state, ord("="));
	    $dataSource = $state->{lexeme};
	    match($state, STRING);
	} else {
	    push @$config, parse_perDB($state);
	}
    }

    my $hashref = { map { $_->{dbname}, $_ } @$config };
    $hashref->{"*dataSource"} = $dataSource # treat as a special database name
	or die "no dataSource specified";

    return bless $hashref, $state->{class};
}


### Interaction between parse_perDB() and parse_section() is not good.
# They should probably be merged into a single function.
sub parse_perDB {
    my($state) = @_;

    match($state, DATABASE);
    my $dbname = $state->{lexeme};
    match($state, STRING);
    match($state, ord("{"));
    my $sections = [];
    while ($state->{token} != ord("}")) {
	push @$sections, parse_section($state);
    }
    match($state, ord("}"));

    # Now we have to make hash-keys for the values we've gathered
    my($tableName, $searchSpec, $dataSpec);
    foreach my $section (@$sections) {
	if (!ref $section) {
	    die "multiple table specifications" if defined $tableName;
	    $tableName = $section;
	} elsif ($section->isa($state->{class} . '::SearchSpec')) {
	    die "multiple search sections" if defined $searchSpec;
	    $searchSpec = $section;
	} elsif ($section->isa($state->{class} . '::DataSpec')) {
	    die "multiple data sections" if defined $dataSpec;
	    $dataSpec = $section;
	} else {
	    die "Wh'appen?";
	}
    }

    return bless {
	dbname => $dbname,
	tablename => $tableName,
	searchSpec => $searchSpec,
	dataSpec => $dataSpec,
    }, $state->{class} . '::PerDB';
}


sub parse_section {
    my($state) = @_;

    if ($state->{token} == TABLE) {
	match($state, TABLE);
	match($state, ord("="));
	my $tableName = $state->{lexeme};
	match($state, STRING);
	return $tableName;
    } elsif ($state->{token} == SEARCH) {
	return parse_searchSpec($state);
    } elsif ($state->{token} == DATA) {
	return parse_dataSpec($state);
    } else {
	# We know this will fail: match(0) always will.
	match($state, 0, "SEARCH or DATA section");
    }
}


sub parse_searchSpec {
    my($state) = @_;

    match($state, SEARCH);
    my $attrsets = [];
    if ($state->{token} == ord("{")) {
	# Multiple attribute-set specifications
	match($state, ord("{"));
	while ($state->{token} != ord("}")) {
	    push @$attrsets, parse_attrsetSpec($state);
	}
	match($state, ord("}"));
    } else {
	# Single attribute-set specification
	push @$attrsets, parse_attrsetSpec($state);
    }

    # Now turn the list of lists into a hash keyed on attrset/type
    my $hashref;
    foreach my $attrset (@$attrsets) {
	my $oid = $attrset->{attrset};
	foreach my $ap (@{ $attrset->{specs} }) {
	    $ap->{attrset} = $oid;
	    $hashref->{"$oid:" . $ap->{accesspoint}} = $ap;
	}
    }

    return bless $hashref, $state->{class} . '::SearchSpec';
}


sub parse_attrsetSpec {
    my($state) = @_;

    match($state, ATTRSET);
    my $attrset = [ $state->{lexeme} ];
    match($state, NUMBER);
    while ($state->{token} == ord(".")) {
	match($state, ord("."));
	push @$attrset, $state->{lexeme};
	match($state, NUMBER);
    }

    my $specs = [];
    match($state, ord("{"));
    while ($state->{token} != ord("}")) {
	push @$specs, parse_accessSpec($state);
    }
    match($state, ord("}"));

    return bless { attrset => join('.', @$attrset), specs => $specs };
}


sub parse_accessSpec {
    my($state) = @_;

    match($state, ACCESS);
    my $accesspoint = $state->{lexeme};
    match($state, NUMBER);
    match($state, ord("="));
    my $columnname = $state->{lexeme};
    match($state, STRING);

    return bless {
	accesspoint => $accesspoint,
	columnname => $columnname,
    }, $state->{class} . '::AccessSpec';
}


sub parse_dataSpec {
    my($state) = @_;

    match($state, DATA);
    my $hashref = {};
    if ($state->{token} == ord("{")) {
	# Multiple record-syntax specifications
	match($state, ord("{"));
	while ($state->{token} != ord("}")) {
	    parse_formatSpec($state, $hashref);
	}
	match($state, ord("}"));
    } else {
	# Single record-syntax specification
	parse_formatSpec($state, $hashref);
    }

    return bless $hashref, $state->{class} . '::DataSpec';
}


# Note that this messes with %$hashref and does not return anything
sub parse_formatSpec {
    my($state, $hashref) = @_;

    # Special case: treat as a magic format-name
    if ($state->{token} == CACHESIZE) {
	match($state, CACHESIZE);
	match($state, ord("="));
	$hashref->{"*cacheSize"} = $state->{lexeme};
	match($state, NUMBER);
	return;
    }

    match($state, FORMAT);
    if ($state->{token} == GRS1) {
	match($state, GRS1);
	$hashref->{GRS1} = parse_GRS1Spec($state);
    } elsif ($state->{token} == XML) {
	match($state, XML);
	$hashref->{XML} = parse_XMLSpec($state);
    } else {
	match($state, GRS1, "format type: GRS1 or XML");
    }
}


sub parse_GRS1Spec {
    my($state) = @_;

    match($state, ord("{"));
    my $specs = [];
    while ($state->{token} != ord("}")) {
	push @$specs, parse_GRS1ElementSpec($state);
    }
    match($state, ord("}"));
    my $hashref = { map { $_->{columnname}, $_ } @$specs };
    return bless $hashref, $state->{class} . '::GRS1Spec';
}


sub parse_GRS1ElementSpec {
    my($state) = @_;

    match($state, COLUMN);
    my $columnname = $state->{lexeme};
    match($state, STRING);
    match($state, ord("="));
    my $tagpath = parse_tagPath($state);

    return bless {
	columnname => $columnname,
	tagpath => $tagpath,
    }, $state->{class} . '::GRS1ElementSpec';
}


sub parse_tagPath {
    my($state) = @_;

    my $tags = [];
    while ($state->{token} == ord("(")) {
	match($state, ord("("));
	my $type = $state->{lexeme};
	match($state, NUMBER);
	match($state, ord(","));
	my $value = $state->{lexeme};
	if ($state->{token} != NUMBER &&
	    $state->{token} != STRING) {
	    match($state, 0, "STRING or NUMBER tag-value");
	}
	match($state, $state->{token});
	match($state, ord(")"));
	push @$tags, [ $type, $value ];
    }
    return bless $tags, $state->{class} . '::TagPath';
}


sub parse_XMLSpec {
    my($state) = @_;

    match($state, ord("{"));
    my $specs = [];
    while ($state->{token} != ord("}")) {
	push @$specs, parse_XMLElementSpec($state);
    }
    match($state, ord("}"));
    my $hashref = { map { $_->{columnname}, $_ } @$specs };
    return bless $hashref, $state->{class} . '::XMLSpec';
}


sub parse_XMLElementSpec {
    my($state) = @_;

    my $columnname;
    if ($state->{token} == RECORD) {
	match($state, RECORD);
	$columnname = "*record"; # treat record-name as a special column
    } else {
	match($state, COLUMN);
	$columnname = $state->{lexeme};
	match($state, STRING);
    }

    match($state, ord("="));
    my $tagname = $state->{lexeme};
    match($state, STRING);

    return bless {
	columnname => $columnname,
	tagname => $tagname,
    }, $state->{class} . '::XMLElementSpec';
}


# PRIVATE to lex()
my %keywords = (database => DATABASE,
		search => SEARCH,
		access => ACCESS,
		attr => ATTR,
		attrset => ATTRSET,
		data => DATA,
		column => COLUMN,
		format => FORMAT,
		grs1 => GRS1,
		xml => XML,
		record => RECORD,
		datasource => DATASOURCE,
		cachesize => CACHESIZE,
		table => TABLE,
		);

my %invert = reverse %keywords;
$invert{STRING()} = "STRING";
$invert{NUMBER()} = "NUMBER";
$invert{SYMBOL()} = "SYMBOL";


sub match {
    my($state, $token, $msg) = @_;

    if ($state->{token} != $token) {
	# Really throwing an exception
	if (defined $msg) {
	    $token = $msg;
	} else {
	    $token = "'" . ($invert{$token} || chr($token)) . "'" .
		" (" . $token . ")";
	}

	# "\n" at the end here prevents Perl from appending the
	# unhelpful text "at blib/lib/Net/Z3950/DBIServer/Config.pm
	# line 414, <GEN0> chunk 9" (or similar) which, when provided,
	# confuses Emacs's M-x next-error parser.
	die($state->{filename} . ":" .
	    $state->{line} . ": " .
	    "expected $token, got '" . $state->{lexeme} . "'" .
	    " (" . $state->{token} . ")\n");
    }

    lex($state);
}


sub lex {
    my($state) = @_;
    my $token = _lex($state);
    $state->{token} = $token;
    #warn "lexed $token (" . $state->{lexeme} . ")";
}


sub _lex {
    my($state) = @_;

    $state->{buffer} =~ s/^\s+//;
    while (!$state->{buffer}) {
	if (!defined ($state->{buffer} = $state->{fh}->getline())) {
	    return undef;
	}
	$state->{line}++;
	#warn "read line " . $state->{line} . ": " . $state->{buffer};
	$state->{buffer} =~ s/#.*$//;
	$state->{buffer} =~ s/\s+$//;
	$state->{buffer} =~ s/^\s+//;
    }

    my $q = '"';		# this avoids confusing emacs's Perl mode
    if ($state->{buffer} =~ s/^"([^$q]*)"//) {
	$state->{lexeme} = $1;
	return STRING
    } elsif ($state->{buffer} =~ s/^([0-9]+)//) {
        $state->{lexeme} = $1;
	return NUMBER;
    } elsif ($state->{buffer} =~ s/^([_a-z][a-z_0-9]*)//i) {
	$state->{lexeme} = $1;
	return $keywords{lc($1)} || SYMBOL;
    }

    # Must be a single character
    $state->{buffer} =~ s/^(.)//;
    $state->{lexeme} = $1;
    return ord($1);
}


=head1 AUTHOR

Mike Taylor E<lt>mike@miketaylor.org.ukE<gt>

First version Sunday 3rd February 2002.

=head1 SEE ALSO

C<Net::Z3950::DBIServer>
is the module that uses this, and the only one that would ever want
to, I'm sure.

You might like to look at
C<Parse::RecDescent>
but I didn't :-)

=cut



# And now, a few undocumented (because they're obvious) utility methods
package Net::Z3950::DBIServer::Config;

sub forDb {
    my $this = shift();
    my($dbname) = @_;
    return $this->{$dbname};
}

sub dataSource {
    my $this = shift();
    return $this->{"*dataSource"};
}


package Net::Z3950::DBIServer::Config::PerDB;

sub dbname { my $this = shift(); return $this->{dbname} }
sub tablename { my $this = shift(); return $this->{tablename} }
sub searchSpec { my $this = shift(); return $this->{searchSpec} }
sub dataSpec { my $this = shift(); return $this->{dataSpec} }


package Net::Z3950::DBIServer::Config::SearchSpec;

sub accessPoint {
    my $this = shift();
    my($attrset, $attrval) = @_;

    return $this->{"$attrset:$attrval"};
}


package Net::Z3950::DBIServer::Config::DataSpec;

sub cacheSize {
    my $this = shift();

    my $cacheSize = $this->{"*cacheSize"};
    $cacheSize = 10 if !defined $cacheSize;
    return $cacheSize
}


package Net::Z3950::DBIServer::Config::XMLSpec;

sub recordName {
    my $this = shift();

    return $this->{"*record"}->{tagname};
}

sub fieldNames {
    my $this = shift();

    my %fields = %$this;
    delete $fields{"*record"};
    return sort keys %fields;
}


1;
