# $Header: /home/cvsroot/zSQLgate/DBIServer/ResultSet.pm,v 1.3 2002/03/03 23:24:22 mike Exp $

package Net::Z3950::DBIServer::ResultSet;
use  Net::Z3950::DBIServer::Exception;
use strict;

=head1 NAME

Net::Z3950::DBIServer::ResultSet - in-progress result set objects

=head1 SYNOPSIS

	$rs = new Net::Z3950::DBIServer::ResultSet($this->{dbh}, "*",
	                                           $SQLcond, $config);
	$n = $rs->count();
	for ($i = 0; $i < $n; $i++) {
	    $hashref = $rs->fetch($i);
	}

=head1 DESCRIPTION

Represents a result set in the process of being inspected by a client.
The initial search is executed at construction time, and the number of
record satisfying the condition stored and made available.
Thereafter, records may be fetched via a random-access interface.

In truth, records must be fetched in order - that's how the DBI
interface, and RDBMSs in general work - and that's how we expect
ResultSets to be used most of the time; but we can give the illusion
of random access by reading ahead to the required record if asked for
one further ahead than the next; and by caching a few of the more
recently-read record after reading them, to support requests for
earlier records.  The upshot is that code like

	for ($i = $n-1; $i >= 0; $i++) {
	    $hashref = $rs->fetch($i);
	}

will work provided that I<$n> is not ``too large'', which means larger
than the implementation-defined record cache associated with the
result set.

=head1 METHODS

=head2 new()

	$rs = new Net::Z3950::DBIServer::ResultSet($dbh, $fields,
						   $cond, $config);

Creates and returns a new C<ResultSet> for records from the DBI
connection handle I<$dbh>, containing the specified I<$fields>
(expressed as a comma-separated list), satisfying the condition
I<$cond> and interpreted by the database-specific configuration
segment I<$config>,

I<$config> must be a reference to a
C<Net::Z3950::DBIServer::Config::PerDB> object - one of the objects in
the array returned as a C<Net::Z3950::DBIServer::Config> object from
that class's constructor.

=cut

sub new {
    my $class = shift();
    my($dbh, $fields, $cond, $config) = @_;

    my $table = $config->tablename();

    # Bah!  Need separate query to discover size of main one.
    my $sql = "SELECT COUNT(*) FROM $table WHERE $cond";
    my $countref = $dbh->selectall_arrayref($sql);
    if (!defined $countref) {
	# Might be a relatively benign condition like searching for a
	# string value in a numeric field, so be tolerant.  229 (Term
	# type not supported) is appealing, but not really legit,
	# since we don't truthfully know what's gone wrong.
	die new Net::Z3950::DBIServer::Exception(1, $dbh->errstr());
    }
    my $count = $countref->[0]->[0];

    $sql = "SELECT $fields FROM $table WHERE $cond";
    #warn "sql=$sql";
    my $sth = $dbh->prepare($sql);
    $sth->execute();

    return bless {
	config => $config,
	count => $count,
	sth => $sth,
	cache => [],
    }, $class;
}


=head2 config()

	$config = $rs->config();

Returns the database-specific configuration segment with which the
result set was created.

=cut

sub config {
    my $this = shift();

    return $this->{config};
}


=head2 count()

	$config = $rs->count();

Returns the number of records in the result set.

=cut

sub count {
    my $this = shift();

    return $this->{count};
}


=head2 fetch()

	$hashref = $rs->fetch($offset);
	foreach $key (keys %$hashref) {
		print($key, "->", $hashref->{$key}, "\n");
	}

Returns a reference to hash mapping fieldnames to values for record
number I<$offset> (counting starts at 0), if that information is
known.  If it can't be obtained (most likely because the underlying
DBI statement handle has already read past the requested record, and
it's fallen out of the result set's cache), then an Exception is
thrown.

=cut

sub fetch {
    my $this = shift();
    my($offset) = @_;

    if ($offset < 0 || $offset >= $this->{count}) {
	die new Net::Z3950::DBIServer::Exception(13);
    }

    my $cache = $this->{cache};
    my $hashref = $cache->[$offset];
    return $hashref
	if defined $hashref;

    if ($offset < @$cache) {
	# Request for a record earlier in the set that the last one.
	# We must have had it earlier, but discarded it because the
	# cache was getting too big.  The only way to get it back now
	# would be by re-executing the query, which might yield
	# different results this time around.  Diagnostic 33
	# (Resources exhausted - valid subset of results available) is
	# not perfect, but it's the best I can find.
	die new Net::Z3950::DBIServer::Exception(33);
    }

    my $sth = $this->{sth};
    while (@$cache <= $offset) {
	my $hashref = $sth->fetchrow_hashref();
	push @$cache, $hashref;

	my $cacheSize = $this->config()->dataSpec()->cacheSize();
	if ($cacheSize != 0 && @$cache > $cacheSize) {
	    my $dropIndex = @$cache-$cacheSize-1;
	    undef $cache->[$dropIndex];
	}
    }

    return $cache->[$offset];
}


=head1 AUTHOR

Mike Taylor E<lt>mike@miketaylor.org.ukE<gt>

First version Saturday 23rd February 2002.

=head1 SEE ALSO

C<Net::Z3950::DBIServer>
is the module that uses this.

=cut


1;
