# $Header: /home/cvsroot/zSQLgate/DBIServer/Spec.pm,v 1.4 2002/03/05 00:35:41 mike Exp $

package Net::Z3950::DBIServer::Spec;
use strict;


=head1 NAME

Net::Z3950::DBIServer::Spec - specification of zSQLgate configuration file


=head1 SYNOPSIS

C<zSQLgate> configuration files are free-form and treat all whitespace
equivalently, with the sole exception of comments.  Related directives
are grouped together with braces, with sections of detailed
information nested within more general sections.

=head1 DESCRIPTION

Comments are introduced by a hash character (C<#>), and run to the end
of the line.  They are ignored.  Comments are the only part of the
configuration file syntax that treat lines specially: otherwise the
syntax is totally free-form.

The contents of the file must follow this BNF grammar, where C<config>
is the start production and C<@> represents the empty production.

	config		= @
			| chunk config

	chunk		= dataSourceSpec
			| perDb

	dataSourceSpec	= DATASOURCE '=' DSNAME

	perDB		= DATABASE DBNAME '=' TABLENAME '{' section* '}'

	section		= searchSpec
			| dataSpec

	searchSpec	= SEARCH attrsetSpec
			| SEARCH '{' attrsetSpec* '}'

	attrsetSpec	= ATTRSET oid '{' accessSpecs* '}'

	oid		= NUMBER
			| NUMBER '.' oid

	accessSpec	= ACCESS ATTR '=' COLUMNNAME

	dataSpec	= CACHESIZE NUMBER
			| DATA formatSpec
			| DATA '{' formatSpec* '}'

	formatSpec	= FORMAT GRS1 grsFormatSpec
			| FORMAT XML '{' xmlFormatSpec '}'

	grsFormatSpec	= '{' grs1ElementSpec* '}'

	grs1ElementSpec	= COLUMN COLUMNNAME '=' tag-path

	tag-path	= tag
			| tag tag-path

	tag		= '(' NUMBER ',' tag-val ')'

	tag-val		= NUMBER
			| STRING

	xmlFormatSpec	= '{' xmlElementSpec* '}'

	xmlElementSpec	= xmlRecordSpec
			| xmlColumnSpec

	xmlRecordName	= RECORD '=' RECORDNAME

	xmlColumnSpec	= COLUMN COLUMNNAME '=' TAGNAME

	DSNAME		= STRING
	DBNAME		= STRING
	TABLENAME	= STRING
	COLUMNNAME	= STRING
	RECORDNAME	= STRING
	TAGNAME		= STRING
	ATTR		= NUMBER

I make every effort to keep this BNF file up to date with respect to
the actual parsing code in C<Net::Z3950::DBIServer::Config> but I'm
not going to write a lot of prose describing what the various sections
mean yet, because I think this format is likely to change a fair bit
before it stabilises enough for that to be worthwhile.  See the next
section for examples.


=head1 FUTURE DIRECTIONS

I anticipate extending the configuration file syntax in the following
ways:

=head2 Enhancements to Searching

=over 4

=item *

Allow attribute sets to be specified by symbolic name (e.g. C<bib1>)
as well as by OID (e.g. C<1.2.840.10003.3.1>)

Z<>

=item *

Allow Functional and Semantic Qualifiers (as specified in the
Attribute Architecture) to work along with the Use Attributes which
are currently specified to select the access point.  I'm not at all
sure yet how this might look in the config file.

Z<>

=item *

Allow specification of semantics for attributes other than access
points.  (I think that relation, truncation and one or two others can
and should be handled by hardwired code that knows about the relevant
attributes in the BIB-1 and Utility sets.)

Z<>

=back

=head2 Enhancements to Retrieval

=over 4

=item *

Allow GRS-1 data specifications to indicate which schema is in use.
This generalises to allowing records to include arbitrary constant
data - in this case, a schema OID associated with the tag-path (1,1).
If you don't know what that means, it's probably because you prefer
XML to GRS-1 :-)

Z<>

=item *

Allow XML data specifications to indicate which DTD (or schema?) is in
use.  Again, I think this generalises to the inclusion of arbitrary
text - maybe just at the start of the record, but maybe we can
generalise the generalisation.

Z<>

=item *

The current XML specification is almost certainly not expressive
enough.  Not sure what to use instead.  Is there already some sort of
standard for this kind of thing, as (for example) XPath standardises
access to sub-documents of XML documents?

Z<>

=item *

Add support for more record syntaxes, especially the various MARC
formats.

Z<>

=item *

Allow one record syntax to be nominated as the default for when we
can't do the one the client asks for.

Z<>

=item *

Allow elements to be marked for inclusion in the brief record
(element-set name C<b>).  More generally, provide the wherewithal for
the configuration file to specify the contents of arbitrary element
sets.

Z<>

=item *

Provide the means for the contents of field (in any format) to be
specified in terms of a link to another table - so, that for example,
an C<album> record can have a C<artist_name> field, implemented with
C<SELECT name FROM artist WHERE id => I<artist_id>.

Z<>

=back

=head2 Miscellaneous Enhancements

=over 4

=item *

Allow the configuration file to override the back-end data-source (DBI
database, or DSN) on a per-logical-database basis.  Also, set other
DBI parameters such as C<auto_commit>.

Z<>

=item *

Allow the configuration file to specify which port or ports the server
should listen on.  Maybe also other Yaz server options.

Z<>

=item *

Allow specification of per-database authentication requirements.

Z<>

=item *

Allow the inclusion of sub-files.

Z<>

=item *

Provide the ability to make multiple back-end databases look like as a
single large data repository (``union catalogue'').  This is a much
bigger deal the most of the other enhancements mentioned here, and
will probably only happen if there's a real need for it in a specific
project.

Z<>

=back

And no doubt many others that I'll think of later.


=head1 AUTHOR

Mike Taylor E<lt>mike@tecc.co.ukE<gt>

First version Sunday 24th February 2001.

=cut

1;
