# $Header: /home/cvsroot/zSQLgate/DBIServer/Tutorial.pm,v 1.5 2002/03/05 00:35:41 mike Exp $

package Net::Z3950::DBIServer::Tutorial;
use strict;


=head1 NAME

Net::Z3950::DBIServer::Tutorial - tutorial for zSQLgate


=head1 SYNOPSIS

server$ B<psql template1>

	template1=> create database music;
	template1=> \connect music
	music=> create table band(id int, name text);
	music=> create table album(id int, artist_id int, name text,
	        theme text, year int, notes text);

server$ B<cat trivial.nzd>

	datasource = "DBI:Pg:dbname=music"
	database "artists" {
	    table = "band"
	    search attrset 1.2.840.10003.3.1 {
		access 12 = "id"
		access 1 = "name"
	    }
	    data format xml {
		record = "artist"
		column "id" = "artistId"
		column "name" = "artistName"
	    }
	}

server$ B<zSQLgate trivial.nzd tcp:@:3950>

	22:14:17-26/02: log: [log] Adding dynamic Z3950 listener on tcp:@:3950
	22:14:17-26/02: log: [log] Starting server zSQLgate pid=24435
	22:14:17-26/02: log: [log] Entering event loop.
	...

client$ B<yaz-client tcp:@:3950>

	Z> base artists
	Z> find @attr 1=12 @attr 2=1 3
	Number of hits: 2
	Z> format xml
	Z> show 1+2
	<artist>
	 <id>1</id>
	 <name>Pink Floyd</name>
	</artist>
	<artist>
	 <id>2</id>
	 <name>Genesis</name>
	</artist>


=head1 DESCRIPTION

This document is a tutorial for C<zSQLgate>, the Z39.50 server for
relational databases.

If you don't know what C<zSQLgate> is, then you should start by
reading
L<Net::Z3950::DBIServer::Intro>,
the introduction to C<zSQLgate>, and
L<Net::Z3950::DBIServer::LICENCE>,
the licence under which it is available.

This tutorial informall walks through the main features of
C<zSQLgate>, and for many people will be the only part of the
C<zSQLgate> documentation that they ever read.  That's fine - much of
what you need to know will be here.  But please be aware that some of
the more esoteric features will not be discussed here: if you don't
find what you want, you may need to dig into two more rigorous
documents that are supplied with this tutorial:

=over 4

=item *

L<Net::Z3950::DBIServer::Run>
Describes the command-line options available to tailor the behaviour
of C<zSQLgate> at run-time.

Z<>

=item *

L<Net::Z3950::DBIServer::Spec>
Describes the C<zSQLgate> configuration file format in more detail
than this document, explaining the precise meaning of its various
directives.

Z<>

=back

In general, those two documents describe the sorts of things you can
say to C<zSQLgate> - but this one explains why you might want to say
those things.  They are there to answer I<What> questions; this
tutorial aims to tell you I<How>.


=head1 A WALK THROUGH THE SYNOPSIS

Did you read the synopsis?  I mean, did you I<really> read it?  Or did
you just think to yourself ``That looks a bit long and difficult'' and
skip it?  Yes?  Well then, go back and read it.  I mean it.  If you
invest five minutes in understanding the synopsis, you'll feel much
better about the rest of this document.  It shows the process of
creating a database, a C<zSQLgate> configuration file for that
database, the starting of a Z39.50 server for it, and a session with a
Z39.50 client accessing it.

Let's look at the four sections separately.  They will introduce us to
pretty much all of the make C<zSQLgate> features: everything else is
just refinement of these core facilities.

=head2 Database Creation

server$ B<psql template1>

	template1=> create database music;
	template1=> \connect music
	music=> create table band(id int, name text);
	music=> create table album(id int, artist_id int, name text,
	        theme text, year int, notes text);

So here, we are creating a relational database.  This is the one part
of the whole process that C<zSQLgate> can't really help you with,
because the details of how you create and populate a database are very
much down to the specific RDBMS that you happen to be using.  The
commands above create a tiny but self-contained database called
C<music> in the PostgreSQL database: the actual SQL commands to do
this will be very similar in most or all relational databases, but the
front-end program will be different.

How do you choose which RDBMS to use?  Well, for most projects, this
will be a non-issue: you'll be putting a C<zSQLgate> front-end on an
existing database, so that choice will already have been made for you.
(You also won't need to worry about building the database!)

If you don't already have a database, and you're building one from
scratch, then one good choice is PostgreSQL, which is included in
recent Red Hat Linux distributions, along with support for using it in
PHP3, Perl, Java (via JDBC), C (via native or ODBC interfaces), Perl,
Python and Tcl.  If you're not using Red Hat, you can get it from
http://www.postgresql.org/ - but there are plenty of other good RDBMSs
around, and I'm not really in a position to recommend one above
another.

Anyway.  You need to get a database from somewhere if you're going to
build a Z39.50 interface to it.

=head2 zSQLGate Configuration

server$ B<cat trivial.nzd>

	datasource = "DBI:Pg:dbname=music"
	database "artists" {
	    table = "band"
	    search attrset 1.2.840.10003.3.1 {
		access 12 = "id"
		access 1 = "name"
	    }
	    data format xml {
		record = "artist"
		column "id" = "artistId"
		column "name" = "artistName"
	    }
	}

90% of the work of setting up a C<zSQLgate> server is writing the
configuration file.  Its job is to describe the relational database
that is to be made available, couching its tables in terms of Z39.50
databases, and to express how the columns in those tables are mapped
to access points (for searching) and data elements (for retrieval).

What we have here is just about the simplest possible complete
C<zSQLgate> configuration file.  Apart from specifying the datasource
(roughly, the relational database), it provides mapping details for a
single ``database'' (that is, in relational terms, a single table), to
be known as C<artists> in the Z39.50 world.  The details for that
database consist of a statement of the name of the table that is to
provide its records (C<band>), plus sections about how that database
is to be searched, and how its data is to be formatted.

The search specification says that searching is done using the BIB-1
attribute set (OID 1.2.840.10003.3.1), and that two access-points are
recognised: searches against access point 12 (``Local number'' in the
BIB-1 attribute set) are mapped into C<SELECT> statements on the C<id>
column, and those against access point 1 (``Personal name'') to the
C<name> column.

The data specification says that C<zSQLgate> is to support record
retrieval using the XML format, building records wrapped in an
<C<artist>>...<C</artist>> pair, and that each record should contain
the contents of the C<id> and C<name> columns, wrapped in
<C<artistId>> and C<<artistName>> tags respectively.

See?  It's easy!

Yes, it's possible to say much, much more in a configuration file; but
the idea is that you can start by writing a simple one, and add in the
more sophisticated stuff as you need it, rather than needing to learn
everything in one go.

=head2 Running the zSQLgate server

server$ B<zSQLgate trivial.nzd tcp:@:3950>

	22:14:17-26/02: log: [log] Adding dynamic Z3950 listener on tcp:@:3950
	22:14:17-26/02: log: [log] Starting server zSQLgate pid=24435
	22:14:17-26/02: log: [log] Entering event loop.
	...

Once you've got your configuration file written, it's time to start
the server.  Generally, you'll need to give it two arguments: the name
of the configuration file you want to use, and the ``listener
address'' which specifies which port to receive incoming connections
on.  Generally, this should be C<tcp:@:>I<number>, where I<number> is
the IP port number.  This is conventionally 210 for Z39.50 servers,
but you'll need to pick a number above 1024 if your server is not
running as root, as the lower-number ports are considered to be
``privileged''.  I think it's cute to use port number 3950.

If there's something wrong with your configuration file, the server
will refuse to start up, giving a hopefully useful message to help you
track down what you've done wrong.  If all is well, it will go into a
loop waiting for connections, C<fork()>ing for each incoming client
and serving its requests in a subprocess.  Unless you've turned
logging off, you'll see startup messages similar to those above.

=head2 Connecting to the zSQLgate server

client$ B<yaz-client tcp:@:3950>

	Z> base artists
	Z> find @attr 1=12 @attr 2=1 3
	Number of hits: 2
	Z> format xml
	Z> show 1+2
	<artist>
	 <id>1</id>
	 <name>Pink Floyd</name>
	</artist>
	<artist>
	 <id>2</id>
	 <name>Genesis</name>
	</artist>

Once your C<zSQLgate> server is up, you'll want to connect a Z39.50
client to it to check that it's doing what you intended.  You can use
whatever client you're used to - that's the point of an interoperable
standard like Z39.50.

In this example, I'm using the simple command-line client provided by
with the Yaz toolkit, because it's the one client you're guaranteed to
have.  (You wouldn't have been able to build C<zSQLgate> and the
libraries on which it depends otherwise.)  Start the client with a
single command-line argument which specifies what port to connect to.
(The part in the middle specifies an internet hostname: C<@> is
shorthand for the same host the client's running on.)

Once the client is connected, you can do what you like.  In the short
session above, we've set the name of the database we want to use to
C<artists> (which is the Z39.50 database name that we specified in the
C<zSQLgate> configuration file), then fired off a search in the rather
opaque but general Prefix Query notation.  The search C<@attr 1=12
@attr 2=1 3> says to search for the term C<3> with two attributes
attached: the first, C<1=12>, means to use access point 12 (which we
defined to mean a C<SELECT> on the C<id> column); and the second,
C<2=1> is a relation attribute, meaning to search for values less than
the specified one.  Inside C<zSQLgate>, this search is translated into

	SELECT * FROM band WHERE id < 3

Finally, having received a response to our search (which found two
records, presumably with C<id>s 1 and 2), we set the desired retrieval
format to XML, and fetch the records, which are rendered to the
screen.  (The real Yaz client includes more structural information
than this, but the substance of it is the two tiny XML records listed
above.)

=head2 Conclusion

An ounce of example is worth a ton of explanation, isn't it?


=head1 THE REST OF THE DOCUMENT

The rest of this tutorial remains to be written, but I though that the
walk through the synopsis provided enough of a taster that it was not
worth delaying the release of the module until I'd finished it.

Here's where we're headed:

=head2 Setting up a Sample Database

I<###> Not yet written

=head2 Configuration File Comments

I<###> Not yet written

=head2 DBI and the Datasource Specification

I<###> Not yet written

=head2 Supporting Multiple Databases

I<###> Not yet written

=head2 Specifying the Search Behaviour

I<###> Not yet written

=head2 Searching with Multiple Attribute Sets

I<###> Not yet written

=head2 XML Record Support

I<###> Not yet written

=head2 GRS-1 Record Support

I<###> Not yet written

=head2 Providing Multiple Record Formats

I<###> Not yet written

=head2 Specifying Cache Size

I<###> Not yet written

=head2 Putting it Together

Here, then, is a more substantial configuration file using all of the
facilities we've discussed:

	# music.nzd - a sample configuration file for Net::Z39.50::DBIServer
	# $Header: /home/cvsroot/zSQLgate/DBIServer/Tutorial.pm,v 1.5 2002/03/05 00:35:41 mike Exp $
	#
	# See Net::Z3950::DBIServer::Spec for a formal description of this
	# file's format, if you need it.  But it's actually pretty obvious.

	datasource = "DBI:Pg:dbname=music"

	database "artists" {
	    table = "band"
	    search {
		attrset 1.2.840.10003.3.1 { # BIB-1
		    access 12 = "id"		# local number
		    access 1 = "name"		# author
		    # 4=title and 21=subject not used in this table
		}
	    }
	    data {
		cachesize = 3
		format grs1 {
		    column "id" = (1,14)	# tagSet-M local control number
		    column "name" = (2,2)	# tagSet-G author
		}
		format xml {
		    record = "artist"
		    column "id" = "artistId"
		    column "name" = "artistName"
		}
	    }
	}

	database "works" {
	    table = "album"
	    search {
		attrset 1.2.840.10003.3.1 { # BIB-1
		    access 12 = "id"		# local number
		    access 4 = "name"		# title
		    access 21 = "theme"		# subject
		    access 30 = "year"		# year
		}
		attrset 1.2.840.10003.3.13 { # Zthes-1
		    access 4 = "year"		# daft test for multiple attrsets
		}
	    }
	    data {
		cachesize = 2
		format grs1 {
		    column "id" = (1,14)	# tagSet-M local control number
		    column "band_id" = (3,"artistId")
		    column "name" = (2,1)	# tagSet-G title
		    column "theme" = (2,21)	# tagSet-G subject
		    column "year" = (2,4)	# tagSet-G date
		    column "notes" = (3,"local")(3,"notes")
		}
		format xml {
		    record = "album"
		    column "id" = "albumId"
		    column "band_id" = "artistId"
		    column "name" = "albumName"
		    column "theme" = "albumTheme"
		    column "year" = "albumYear"
		    column "notes" = "albumNotes"
		}
	    }
	}


=head1 AUTHOR

Mike Taylor E<lt>mike@tecc.co.ukE<gt>

First version Sunday 24th February 2001.

=cut

1;
